package jdk.internal.org.objectweb.asm.commons;

import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Type;
import java.util.HashMap;
import java.util.ArrayList;
import jdk.internal.org.objectweb.asm.MethodVisitor;
import jdk.internal.org.objectweb.asm.Label;
import java.util.Map;
import java.util.List;
import jdk.internal.org.objectweb.asm.Opcodes;

public abstract class AdviceAdapter extends GeneratorAdapter implements Opcodes
{
    private static final Object THIS;
    private static final Object OTHER;
    protected int methodAccess;
    protected String methodDesc;
    private boolean constructor;
    private boolean superInitialized;
    private List<Object> stackFrame;
    private Map<Label, List<Object>> branches;
    
    protected AdviceAdapter(int n, MethodVisitor methodVisitor, int methodAccess, String anObject, String methodDesc) {
        super(n, methodVisitor, methodAccess, anObject, methodDesc);
        this.methodAccess = methodAccess;
        this.methodDesc = methodDesc;
        this.constructor = "<init>".equals(anObject);
    }
    
    @Override
    public void visitCode() {
        this.mv.visitCode();
        if (this.constructor) {
            this.stackFrame = new ArrayList();
            this.branches = new HashMap();
        }
        else {
            this.superInitialized = true;
            this.onMethodEnter();
        }
    }
    
    @Override
    public void visitLabel(Label label) {
        this.mv.visitLabel(label);
        if (this.constructor && this.branches != null) {
            List<Object> stackFrame = (List)this.branches.get(label);
            if (stackFrame != null) {
                this.stackFrame = stackFrame;
                this.branches.remove(label);
            }
        }
    }
    
    @Override
    public void visitInsn(int opcode) {
        if (this.constructor) {
            switch (opcode) {
                case 177: {
                    this.onMethodExit(opcode);
                    break;
                }
                case 172:
                case 174:
                case 176:
                case 191: {
                    this.popValue();
                    this.onMethodExit(opcode);
                    break;
                }
                case 173:
                case 175: {
                    this.popValue();
                    this.popValue();
                    this.onMethodExit(opcode);
                    break;
                }
                case 0:
                case 47:
                case 49:
                case 116:
                case 117:
                case 118:
                case 119:
                case 134:
                case 138:
                case 139:
                case 143:
                case 145:
                case 146:
                case 147:
                case 190: {
                    break;
                }
                case 1:
                case 2:
                case 3:
                case 4:
                case 5:
                case 6:
                case 7:
                case 8:
                case 11:
                case 12:
                case 13:
                case 133:
                case 135:
                case 140:
                case 141: {
                    this.pushValue(AdviceAdapter.OTHER);
                    break;
                }
                case 9:
                case 10:
                case 14:
                case 15: {
                    this.pushValue(AdviceAdapter.OTHER);
                    this.pushValue(AdviceAdapter.OTHER);
                    break;
                }
                case 46:
                case 48:
                case 50:
                case 51:
                case 52:
                case 53:
                case 87:
                case 96:
                case 98:
                case 100:
                case 102:
                case 104:
                case 106:
                case 108:
                case 110:
                case 112:
                case 114:
                case 120:
                case 121:
                case 122:
                case 123:
                case 124:
                case 125:
                case 126:
                case 128:
                case 130:
                case 136:
                case 137:
                case 142:
                case 144:
                case 149:
                case 150:
                case 194:
                case 195: {
                    this.popValue();
                    break;
                }
                case 88:
                case 97:
                case 99:
                case 101:
                case 103:
                case 105:
                case 107:
                case 109:
                case 111:
                case 113:
                case 115:
                case 127:
                case 129:
                case 131: {
                    this.popValue();
                    this.popValue();
                    break;
                }
                case 79:
                case 81:
                case 83:
                case 84:
                case 85:
                case 86:
                case 148:
                case 151:
                case 152: {
                    this.popValue();
                    this.popValue();
                    this.popValue();
                    break;
                }
                case 80:
                case 82: {
                    this.popValue();
                    this.popValue();
                    this.popValue();
                    this.popValue();
                    break;
                }
                case 89: {
                    this.pushValue(this.peekValue());
                    break;
                }
                case 90: {
                    int size = this.stackFrame.size();
                    this.stackFrame.add(size - 2, this.stackFrame.get(size - 1));
                    break;
                }
                case 91: {
                    int size2 = this.stackFrame.size();
                    this.stackFrame.add(size2 - 3, this.stackFrame.get(size2 - 1));
                    break;
                }
                case 92: {
                    int size3 = this.stackFrame.size();
                    this.stackFrame.add(size3 - 2, this.stackFrame.get(size3 - 1));
                    this.stackFrame.add(size3 - 2, this.stackFrame.get(size3 - 1));
                    break;
                }
                case 93: {
                    int size4 = this.stackFrame.size();
                    this.stackFrame.add(size4 - 3, this.stackFrame.get(size4 - 1));
                    this.stackFrame.add(size4 - 3, this.stackFrame.get(size4 - 1));
                    break;
                }
                case 94: {
                    int size5 = this.stackFrame.size();
                    this.stackFrame.add(size5 - 4, this.stackFrame.get(size5 - 1));
                    this.stackFrame.add(size5 - 4, this.stackFrame.get(size5 - 1));
                    break;
                }
                case 95: {
                    int size6 = this.stackFrame.size();
                    this.stackFrame.add(size6 - 2, this.stackFrame.get(size6 - 1));
                    this.stackFrame.remove(size6);
                    break;
                }
            }
        }
        else {
            switch (opcode) {
                case 172:
                case 173:
                case 174:
                case 175:
                case 176:
                case 177:
                case 191: {
                    this.onMethodExit(opcode);
                    break;
                }
            }
        }
        this.mv.visitInsn(opcode);
    }
    
    @Override
    public void visitVarInsn(int n, int n2) {
        super.visitVarInsn(n, n2);
        if (this.constructor) {
            switch (n) {
                case 21:
                case 23: {
                    this.pushValue(AdviceAdapter.OTHER);
                    break;
                }
                case 22:
                case 24: {
                    this.pushValue(AdviceAdapter.OTHER);
                    this.pushValue(AdviceAdapter.OTHER);
                    break;
                }
                case 25: {
                    this.pushValue((n2 == 0) ? AdviceAdapter.THIS : AdviceAdapter.OTHER);
                    break;
                }
                case 54:
                case 56:
                case 58: {
                    this.popValue();
                    break;
                }
                case 55:
                case 57: {
                    this.popValue();
                    this.popValue();
                    break;
                }
            }
        }
    }
    
    @Override
    public void visitFieldInsn(int opcode, String owner, String name, String descriptor) {
        this.mv.visitFieldInsn(opcode, owner, name, descriptor);
        if (this.constructor) {
            int char1 = descriptor.charAt(0);
            int n = char1 == 'J' || char1 == 'D';
            switch (opcode) {
                case 178: {
                    this.pushValue(AdviceAdapter.OTHER);
                    if (!(n == 0)) {
                        this.pushValue(AdviceAdapter.OTHER);
                        break;
                    }
                    break;
                }
                case 179: {
                    this.popValue();
                    if (!(n == 0)) {
                        this.popValue();
                        break;
                    }
                    break;
                }
                case 181: {
                    this.popValue();
                    if (!(n == 0)) {
                        this.popValue();
                        this.popValue();
                        break;
                    }
                    break;
                }
                default: {
                    if (!(n == 0)) {
                        this.pushValue(AdviceAdapter.OTHER);
                        break;
                    }
                    break;
                }
            }
        }
    }
    
    @Override
    public void visitIntInsn(int opcode, int operand) {
        this.mv.visitIntInsn(opcode, operand);
        if (this.constructor && opcode != 188) {
            this.pushValue(AdviceAdapter.OTHER);
        }
    }
    
    @Override
    public void visitLdcInsn(Object value) {
        this.mv.visitLdcInsn(value);
        if (this.constructor) {
            this.pushValue(AdviceAdapter.OTHER);
            if (value instanceof Double || value instanceof Long) {
                this.pushValue(AdviceAdapter.OTHER);
            }
        }
    }
    
    @Override
    public void visitMultiANewArrayInsn(String descriptor, int numDimensions) {
        this.mv.visitMultiANewArrayInsn(descriptor, numDimensions);
        if (this.constructor) {
            for (int i = 0; i < numDimensions; ++i) {
                this.popValue();
            }
            this.pushValue(AdviceAdapter.OTHER);
        }
    }
    
    @Override
    public void visitTypeInsn(int opcode, String type) {
        this.mv.visitTypeInsn(opcode, type);
        if (this.constructor && opcode == 187) {
            this.pushValue(AdviceAdapter.OTHER);
        }
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, opcode == 185);
        return;
    }
    
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor, isInterface);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, isInterface);
        return;
    }
    
    private void doVisitMethodInsn(int opcode, String owner, String name, String methodDescriptor, boolean isInterface) {
        this.mv.visitMethodInsn(opcode, owner, name, methodDescriptor, isInterface);
        if (this.constructor) {
            Type[] argumentTypes = Type.getArgumentTypes(methodDescriptor);
            for (int i = 0; i < argumentTypes.length; ++i) {
                this.popValue();
                if (argumentTypes[i].getSize() == 2) {
                    this.popValue();
                }
            }
            switch (opcode) {
                case 182:
                case 185: {
                    this.popValue();
                    break;
                }
                case 183: {
                    if (this.popValue() == AdviceAdapter.THIS && !this.superInitialized) {
                        this.onMethodEnter();
                        this.superInitialized = true;
                        this.constructor = false;
                        break;
                    }
                    break;
                }
            }
            Type returnType = Type.getReturnType(methodDescriptor);
            if (returnType != Type.VOID_TYPE) {
                this.pushValue(AdviceAdapter.OTHER);
                if (returnType.getSize() == 2) {
                    this.pushValue(AdviceAdapter.OTHER);
                }
            }
        }
    }
    
    @Override
    public void visitInvokeDynamicInsn(String name, String methodDescriptor, Handle bootstrapMethodHandle, Object... bootstrapMethodArguments) {
        this.mv.visitInvokeDynamicInsn(name, methodDescriptor, bootstrapMethodHandle, bootstrapMethodArguments);
        if (this.constructor) {
            Type[] argumentTypes = Type.getArgumentTypes(methodDescriptor);
            for (int i = 0; i < argumentTypes.length; ++i) {
                this.popValue();
                if (argumentTypes[i].getSize() == 2) {
                    this.popValue();
                }
            }
            Type returnType = Type.getReturnType(methodDescriptor);
            if (returnType != Type.VOID_TYPE) {
                this.pushValue(AdviceAdapter.OTHER);
                if (returnType.getSize() == 2) {
                    this.pushValue(AdviceAdapter.OTHER);
                }
            }
        }
    }
    
    @Override
    public void visitJumpInsn(int opcode, Label label) {
        this.mv.visitJumpInsn(opcode, label);
        if (this.constructor) {
            switch (opcode) {
                case 153:
                case 154:
                case 155:
                case 156:
                case 157:
                case 158:
                case 198:
                case 199: {
                    this.popValue();
                    break;
                }
                case 159:
                case 160:
                case 161:
                case 162:
                case 163:
                case 164:
                case 165:
                case 166: {
                    this.popValue();
                    this.popValue();
                    break;
                }
                case 168: {
                    this.pushValue(AdviceAdapter.OTHER);
                    break;
                }
            }
            this.addBranch(label);
        }
    }
    
    @Override
    public void visitLookupSwitchInsn(Label dflt, int[] keys, Label[] labels) {
        this.mv.visitLookupSwitchInsn(dflt, keys, labels);
        if (this.constructor) {
            this.popValue();
            this.addBranches(dflt, labels);
        }
    }
    
    @Override
    public void visitTableSwitchInsn(int min, int max, Label dflt, Label... labels) {
        this.mv.visitTableSwitchInsn(min, max, dflt, labels);
        if (this.constructor) {
            this.popValue();
            this.addBranches(dflt, labels);
        }
    }
    
    @Override
    public void visitTryCatchBlock(Label start, Label end, Label handler, String type) {
        super.visitTryCatchBlock(start, end, handler, type);
        if (this.constructor && !this.branches.containsKey(handler)) {
            ArrayList<AdviceAdapter> list = new ArrayList();
            list.add(AdviceAdapter.OTHER);
            this.branches.put(handler, list);
        }
    }
    
    private void addBranches(Label label, Label[] array) {
        this.addBranch(label);
        for (int i = 0; i < array.length; ++i) {
            this.addBranch(array[i]);
        }
    }
    
    private void addBranch(Label label) {
        if (this.branches.containsKey(label)) {
            return;
        }
        this.branches.put(label, new ArrayList(this.stackFrame));
        return;
    }
    
    private Object popValue() {
        return this.stackFrame.remove(this.stackFrame.size() - 1);
    }
    
    private Object peekValue() {
        return this.stackFrame.get(this.stackFrame.size() - 1);
    }
    
    private void pushValue(Object o) {
        this.stackFrame.add(o);
    }
    
    protected void onMethodEnter() {
    }
    
    protected void onMethodExit(int n) {
    }
    
    static {
        THIS = new Object();
        OTHER = new Object();
    }
}
